/*
 * Skeleton.cpp
 *
 *  Created on: Jul 28, 2014
 *      Author: coert
 */

#include "Skeleton.h"

namespace nl_uu_science_gmt
{

const JointNameMap Skeleton::JointNames = initJointNames();
const ActionNameMap Skeleton::ActionNames = initActionNames();

void Skeleton::write(cv::FileStorage &fs) const
{
	assert(fs.isOpened());

	fs << "{";
	fs << "id" << (int) s_id;
	fs << "fid" << (int) s_fid;
	fs << "aid" << (int) s_action_class;
	fs << "action_name" << ActionNames.at(s_action_class);
	fs << "joints_amount" << (int) s_points2D.size();
	fs << "joints";
	fs << "{";
	for (auto joint2i : s_points2D)
	{
		std::stringstream joint_name;
		joint_name << "joint_" << joint2i.first;
		fs << joint_name.str();
		fs << "{";
		fs << "id" << (int) joint2i.first;
		fs << "name" << JointNames.at(joint2i.first);
		fs << "point2Di" << joint2i.second;
		auto joint3d = s_points3D.find(joint2i.first);
		if (joint3d != s_points3D.end()) fs << "point3Df" << joint3d->second;
		fs << "}";
	}
	fs << "}";
	fs << "}";
}

void Skeleton::read(const cv::FileNode &fn)
{
	assert(fn.type() == cv::FileNode::MAP);

	fn["id"] >> (int) s_id;
	fn["fid"] >> (int) s_fid;
	int aid = SKELETON_POSITION_COUNT;
	fn["aid"] >> aid;
	s_action_class = (ACTION_CLASSES_INDEX) aid;

	for (cv::FileNodeIterator n = fn["joints"].begin(); n != fn["joints"].end(); ++n)
	{
		assert((*n).type() == cv::FileNode::MAP);

		int id = 0;
		(*n)["id"] >> id;
		cv::Point2i point2Di;
		(*n)["point2Di"] >> point2Di;
		cv::Point3d point3Df;
		(*n)["point3Df"] >> point3Df;

		s_points2D.insert(std::make_pair((NUI_SKELETON_POSITION_INDEX) id, point2Di));
		s_points3D.insert(std::make_pair((NUI_SKELETON_POSITION_INDEX) id, point3Df));
	}
}

void Skeleton::drawBone(const JointPoint2iMap &skeleton, const NUI_SKELETON_POSITION_INDEX &joint0,
		const NUI_SKELETON_POSITION_INDEX &joint1, const cv::Scalar &color, cv::Mat &frame)
{
	auto skeleton_joint0 = skeleton.find(joint0);
	if (skeleton_joint0 != skeleton.end())
	{
		auto skeleton_joint1 = skeleton.find(joint1);
		if (skeleton_joint1 != skeleton.end())
			line(frame, skeleton_joint0->second, skeleton_joint1->second, color, 2, CV_AA);
	}
}

void Skeleton::drawTorso(const JointPoint2iMap &skeleton, const cv::Scalar &color, cv::Mat &frame)
{
	drawBone(skeleton, SKELETON_POSITION_HEAD, SKELETON_POSITION_SHOULDER_CENTER, color, frame);
	drawBone(skeleton, SKELETON_POSITION_SHOULDER_CENTER, SKELETON_POSITION_SHOULDER_LEFT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_SHOULDER_CENTER, SKELETON_POSITION_SHOULDER_RIGHT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_SHOULDER_CENTER, SKELETON_POSITION_SPINE, color, frame);
	drawBone(skeleton, SKELETON_POSITION_SPINE, SKELETON_POSITION_HIP_CENTER, color, frame);
	drawBone(skeleton, SKELETON_POSITION_HIP_CENTER, SKELETON_POSITION_HIP_LEFT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_HIP_CENTER, SKELETON_POSITION_HIP_RIGHT, color, frame);
}

void Skeleton::drawLeftArm(const JointPoint2iMap &skeleton, const cv::Scalar &color, cv::Mat &frame)
{
	drawBone(skeleton, SKELETON_POSITION_SHOULDER_LEFT, SKELETON_POSITION_ELBOW_LEFT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_ELBOW_LEFT, SKELETON_POSITION_WRIST_LEFT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_WRIST_LEFT, SKELETON_POSITION_HAND_LEFT, color, frame);
}

void Skeleton::drawRightArm(const JointPoint2iMap &skeleton, const cv::Scalar &color, cv::Mat &frame)
{
	drawBone(skeleton, SKELETON_POSITION_SHOULDER_RIGHT, SKELETON_POSITION_ELBOW_RIGHT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_ELBOW_RIGHT, SKELETON_POSITION_WRIST_RIGHT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_WRIST_RIGHT, SKELETON_POSITION_HAND_RIGHT, color, frame);
}

void Skeleton::drawLeftLeg(const JointPoint2iMap &skeleton, const cv::Scalar &color, cv::Mat &frame)
{
	drawBone(skeleton, SKELETON_POSITION_HIP_LEFT, SKELETON_POSITION_KNEE_LEFT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_KNEE_LEFT, SKELETON_POSITION_ANKLE_LEFT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_ANKLE_LEFT, SKELETON_POSITION_FOOT_LEFT, color, frame);
}

void Skeleton::drawRightLeg(const JointPoint2iMap &skeleton, const cv::Scalar &color, cv::Mat &frame)
{
	drawBone(skeleton, SKELETON_POSITION_HIP_RIGHT, SKELETON_POSITION_KNEE_RIGHT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_KNEE_RIGHT, SKELETON_POSITION_ANKLE_RIGHT, color, frame);
	drawBone(skeleton, SKELETON_POSITION_ANKLE_RIGHT, SKELETON_POSITION_FOOT_RIGHT, color, frame);
}

void Skeleton::drawSkeleton(const JointPoint2iMap &skeleton, const cv::Scalar &color, cv::Mat &frame)
{
	drawTorso(skeleton, color, frame);
	drawLeftArm(skeleton, color, frame);
	drawRightArm(skeleton, color, frame);
	drawLeftLeg(skeleton, color, frame);
	drawRightLeg(skeleton, color, frame);
}

void Skeleton::drawSkeleton(const cv::Scalar &joint_color, const cv::Scalar &bone_color, cv::Mat &frame)
{
	auto text = ActionNames.at(s_action_class);
	putText(frame, text, s_points2D.at(SKELETON_POSITION_HEAD), CV_FONT_HERSHEY_SIMPLEX, .5, CV_RGB(0, 0, 0), 2, CV_AA);
	putText(frame, text, s_points2D.at(SKELETON_POSITION_HEAD), CV_FONT_HERSHEY_SIMPLEX, .5, CV_RGB(0, 255, 0), 1.5, CV_AA);

	drawSkeleton(s_points2D, bone_color, frame);
	for (auto joint : s_points2D)
		circle(frame, joint.second, 2, joint_color, -1, CV_AA);
}

double Skeleton::calcDirection()
{
	auto ls_shoulder = s_points3D.find(SKELETON_POSITION_SHOULDER_LEFT);
	auto rs_shoulder = s_points3D.find(SKELETON_POSITION_SHOULDER_RIGHT);
	auto ls_hip = s_points3D.find(SKELETON_POSITION_HIP_LEFT);
	auto rs_hip = s_points3D.find(SKELETON_POSITION_HIP_RIGHT);

	if (s_angle == -1 && !(ls_shoulder == s_points3D.end() || rs_shoulder == s_points3D.end()
													|| ls_hip == s_points3D.end() || rs_hip == s_points3D.end()))
	{
		double dx = ls_shoulder->second.x - rs_shoulder->second.x;
		double dz = ls_shoulder->second.z - rs_shoulder->second.z;

		cv::Point3d chest_center;
		chest_center.x = (ls_shoulder->second.x + rs_shoulder->second.x) / 2;
		chest_center.y = (ls_shoulder->second.y + rs_shoulder->second.y) / 2;
		chest_center.z = (ls_shoulder->second.z + rs_shoulder->second.z) / 2;
		cv::Point3d chest_perpendicular;
		chest_perpendicular.x = chest_center.x - (dz / 2);
		chest_perpendicular.y = (ls_shoulder->second.y + rs_shoulder->second.y) / 2;
		chest_perpendicular.z = chest_center.z + (dx / 2);
		double chest_length = cv::norm(chest_perpendicular - chest_center);
		double sq_length_ab = chest_length * chest_length;
		cv::Point3d chest_normal;
		chest_normal.x = chest_center.x - chest_length;
		chest_normal.y = (ls_shoulder->second.y + rs_shoulder->second.y) / 2;
		chest_normal.z = (ls_shoulder->second.z + rs_shoulder->second.z) / 2;
		double norm_chest = cv::norm(chest_normal - chest_perpendicular);
		double sq_length_c = norm_chest * norm_chest;
		double chest_angle = acos(((sq_length_ab + sq_length_ab) - sq_length_c) / (2.0 * chest_length * chest_length));

		dx = ls_hip->second.x - rs_hip->second.x;
		dz = ls_hip->second.z - rs_hip->second.z;

		cv::Point3d hip_center;
		hip_center.x = (ls_hip->second.x + rs_hip->second.x) / 2;
		hip_center.y = (ls_hip->second.y + rs_hip->second.y) / 2;
		hip_center.z = (ls_hip->second.z + rs_hip->second.z) / 2;
		cv::Point3d hip_perpendicular;
		hip_perpendicular.x = hip_center.x - (dz / 2);
		hip_perpendicular.y = (ls_hip->second.y + rs_hip->second.y) / 2;
		hip_perpendicular.z = hip_center.z + (dx / 2);
		double hip_length = cv::norm(hip_perpendicular - hip_center);
		sq_length_ab = hip_length * hip_length;
		cv::Point3d hip_normal;
		hip_normal.x = hip_center.x - hip_length;
		hip_normal.y = (ls_hip->second.y + rs_hip->second.y) / 2;
		hip_normal.z = (ls_hip->second.z + rs_hip->second.z) / 2;
		double norm_hip = cv::norm(hip_normal - hip_perpendicular);
		sq_length_c = norm_hip * norm_hip;
		double hip_angle = acos(((sq_length_ab + sq_length_ab) - sq_length_c) / (2.0 * hip_length * hip_length));

		s_angle = (chest_angle + hip_angle) / 2;
	}

	return s_angle;
}

} /* namespace nl_uu_science_gmt */
