/*
 * Skeleton.h
 *
 *  Created on: Jul 28, 2014
 *      Author: coert
 */

#ifndef SKELETON_H_
#define SKELETON_H_

#include <memory>
#include <opencv2/opencv.hpp>

#include "IO.h"
#include "Types.h"

namespace nl_uu_science_gmt
{

class Skeleton
{
	uint s_id = 0;
	uint s_fid = 0;
	ACTION_CLASSES_INDEX s_action_class = AC_NA;

	cv::Rect s_bbox;
	cv::Point3d s_tlb;
	cv::Point3d s_brf;

	JointPoint2iMap s_points2D;
	JointPoint3dMap s_points3D;

	double s_angle = 0;

	MatPtr s_rotation = nullptr;
	MatPtr s_translation = nullptr;
	MatPtr s_camera_matrix = nullptr;
	MatPtr s_dist_coeff = nullptr;

	static inline const JointNameMap initJointNames()
	{
		JointNameMap jointNameStringMapping;
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HIP_CENTER,      "hip"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_SPINE,           "spine"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_SHOULDER_CENTER, "shoulder"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HEAD,            "head"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_SHOULDER_LEFT,   "left_shoulder"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_ELBOW_LEFT,      "left_elbow"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_WRIST_LEFT,      "left_wrist"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HAND_LEFT,       "left_hand"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_SHOULDER_RIGHT,  "right_shoulder"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_ELBOW_RIGHT,     "right_elbow"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_WRIST_RIGHT,     "right_wrist"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HAND_RIGHT,      "right_hand"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HIP_LEFT,        "left_hip"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_KNEE_LEFT,       "left_knee"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_ANKLE_LEFT,      "left_ankle"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_FOOT_LEFT,       "left_foot"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_HIP_RIGHT,       "right_hip"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_KNEE_RIGHT,      "right_knee"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_ANKLE_RIGHT,     "right_ankle"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_FOOT_RIGHT,      "right_foot"));
		jointNameStringMapping.insert(std::make_pair(SKELETON_POSITION_COUNT,           "_count_"));

		return jointNameStringMapping;
	}

	static inline const ActionNameMap initActionNames()
	{
		ActionNameMap actionNameStringMapping;
		actionNameStringMapping.insert(std::make_pair(AC_STANDING,    "standing"));
		actionNameStringMapping.insert(std::make_pair(AC_APPROACHING, "approaching"));
		actionNameStringMapping.insert(std::make_pair(AC_HANDSHAKING, "handshake"));
		actionNameStringMapping.insert(std::make_pair(AC_HIGHFIVE,    "high-five"));
		actionNameStringMapping.insert(std::make_pair(AC_LEAVING,     "leaving"));
		actionNameStringMapping.insert(std::make_pair(AC_END,         "end"));
		actionNameStringMapping.insert(std::make_pair(AC_NA,          "_count_"));

		return actionNameStringMapping;
	}

	static inline void drawBone(const JointPoint2iMap &,
			const NUI_SKELETON_POSITION_INDEX &, const NUI_SKELETON_POSITION_INDEX &, const cv::Scalar &, cv::Mat &);
	static inline void drawTorso(const JointPoint2iMap &, const cv::Scalar &, cv::Mat &);
	static inline void drawLeftArm(const JointPoint2iMap &, const cv::Scalar &, cv::Mat &);
	static inline void drawRightArm(const JointPoint2iMap &, const cv::Scalar &, cv::Mat &);
	static inline void drawLeftLeg(const JointPoint2iMap &, const cv::Scalar &, cv::Mat &);
	static inline void drawRightLeg(const JointPoint2iMap &, const cv::Scalar &, cv::Mat &);
	static inline void drawSkeleton(const JointPoint2iMap &, const cv::Scalar &, cv::Mat &);

public:
	static const JointNameMap JointNames;
	static const ActionNameMap ActionNames;

	void write(cv::FileStorage &) const;
	void read(const cv::FileNode &);

	void drawSkeleton(const cv::Scalar &, const cv::Scalar &, cv::Mat &);
	double calcDirection();

	const uint getId() const
	{
		return s_id;
	}

	void setId(uint id)
	{
		s_id = id;
	}

	const uint getFid() const
	{
		return s_fid;
	}

	void setFid(uint fid)
	{
		s_fid = fid;
	}

	double getAngle() const
	{
		return s_angle;
	}

	void setAngle(double angle)
	{
		s_angle = angle;
	}

	const cv::Rect& getBbox() const
	{
		return s_bbox;
	}

	void setBbox(const cv::Rect& bbox)
	{
		s_bbox = bbox;
	}

	const cv::Point3d& getBrf() const
	{
		return s_brf;
	}

	void setBrf(const cv::Point3d& brf)
	{
		s_brf = brf;
	}

	const MatPtr& getCameraMatrix() const
	{
		return s_camera_matrix;
	}

	void setCameraMatrix(const MatPtr& cameraMatrix)
	{
		s_camera_matrix = cameraMatrix;
	}

	const MatPtr& getDistCoeff() const
	{
		return s_dist_coeff;
	}

	void setDistCoeff(const MatPtr& distCoeff)
	{
		s_dist_coeff = distCoeff;
	}

	const JointPoint2iMap& getPoints2D() const
	{
		return s_points2D;
	}

	void setPoints2D(const JointPoint2iMap& points2D)
	{
		s_points2D = points2D;
	}

	const JointPoint3dMap& getPoints3D() const
	{
		return s_points3D;
	}

	void setPoints3D(const JointPoint3dMap& points3D)
	{
		s_points3D = points3D;
	}

	const MatPtr& getRotation() const
	{
		return s_rotation;
	}

	void setRotation(const MatPtr& rotation)
	{
		s_rotation = rotation;
	}

	const cv::Point3d& getTlb() const
	{
		return s_tlb;
	}

	void setTlb(const cv::Point3d& tlb)
	{
		s_tlb = tlb;
	}

	const MatPtr& getTranslation() const
	{
		return s_translation;
	}

	void setTranslation(const MatPtr& translation)
	{
		s_translation = translation;
	}

	const ACTION_CLASSES_INDEX getActionClass() const
	{
		return s_action_class;
	}

	void setActionClass(ACTION_CLASSES_INDEX actionClass)
	{
		s_action_class = actionClass;
	}
};

} /* namespace nl_uu_science_gmt */

#endif /* SKELETON_H_ */
