/*
 * Video.cpp
 *
 *  Created on: Jul 28, 2014
 *      Author: coert
 */

#include "Video.h"

namespace nl_uu_science_gmt
{

void Video::write(cv::FileStorage &fs) const
{
	assert(fs.isOpened());

	fs << "{";
	fs << "id" << (int) s_id;
	fs << "image_path" << s_image_path;
	fs << "video_size" << s_size;
	if (s_camera_matrix != nullptr) fs << "camera_matrix" << *s_camera_matrix;
	if (s_dist_coeff != nullptr) fs << "dist_coeff" << *s_dist_coeff;
	fs << "frames_amount" << (int) s_frames.size();
	if(!s_frames.empty())
	{
		fs << "frames";
		fs << "{";
		for (auto frame : s_frames)
		{
			std::stringstream frame_name;
			frame_name << "frame_" << frame->getId();
			fs << frame_name.str() << *frame;
		}
		fs << "}";
	}
	fs << "}";
}

void Video::read(const cv::FileNode &fn)
{
	assert(fn.type() == cv::FileNode::MAP);

	int id = 0;
	fn["id"] >> id;
	s_id = id;
	fn["image_path"] >> s_image_path;
	fn["video_size"] >> s_size;
	int frames_amount;
	fn["frames_amount"] >> frames_amount;
	if (frames_amount > 0)
	{
		cv::Mat camera_matrix;
		fn["camera_matrix"] >> camera_matrix;
		cv::Mat dist_coeff;
		fn["dist_coeff"] >> dist_coeff;
		for (cv::FileNodeIterator n = fn["frames"].begin(); n != fn["frames"].end();)
		{
			Frame frame;
			n >> frame;
			frame.setVideo(this);

			addFrame(frame);
		}
	}
}

void Video::createVideoCapture(const std::string &filename)
{
	s_video_capture = std::make_shared<cv::VideoCapture>(filename);
}

void Video::addFrame(Frame &frame)
{
	s_frames.push_back(std::make_shared<Frame>(frame));
}

void Video::keyboard(const int key)
{
	switch (key)
	{
	case 27:
		s_stop = !s_stop;
		break;
	case 'n':
	case 'N':
		s_next = !s_next;
		break;
	case 'p':
	case 'P':
		s_paused = !s_paused;
		break;
	}
}

int Video::play()
{
	int current_fid = 0;

	const std::string window_name = "video";
	const std::string trackbar_name = "frame";
	cv::namedWindow(window_name, CV_WINDOW_KEEPRATIO);
	cv::resizeWindow(window_name, s_size.width, s_size.height);
	cv::createTrackbar(trackbar_name, window_name, &current_fid, s_frames.size() - 1);

	while(!(s_stop || s_next))
	{
		auto frame = s_frames[current_fid];

		cv::Mat canvas;
		frame->getImage(canvas);
		for(auto skeleton : frame->getSkeletons())
			skeleton->drawSkeleton(CV_RGB(255, 255, 0), CV_RGB(255, 0, 0), canvas); // Yellow joints, Red bones

		cv::setTrackbarPos(trackbar_name, window_name, current_fid);

		if(!s_paused) ++current_fid;
		if(current_fid >= s_frames.size()) s_next = true;

		cv::imshow("video", canvas);
		keyboard(cv::waitKey(80));
	}

	return (s_stop || s_next) ? s_stop ? 27 : s_next ? 'n' : -1 : -1;
}

} /* namespace nl_uu_science_gmt */
